import readline
import subprocess

from oslui.command.base import BaseCommand


class ShellCommand(BaseCommand):
    """
        ShellCommand: shell command
    """

    def __init__(self, content: str):
        super().__init__(content)

    def modify(self):
        readline.add_history(self.content)

        def startup_hook():
            readline.insert_text(self.content)

        readline.set_startup_hook(startup_hook)
        self.content = input()

    def run(self):
        subprocess.run(self.content, shell=True)



from langchain.chains import LLMChain
from langchain.llms.base import BaseLLM
from langchain.prompts import PromptTemplate

from oslui.agent.base import BaseAgent


class TranslateAgent(BaseAgent):
    """
        TranslateAgent: translate natural language command to shell command
    """
    chain: LLMChain = None

    def __init__(self, llm: BaseLLM, prompt: PromptTemplate = None, memory: PromptTemplate = None):
        super().__init__(llm, prompt, memory)
        self.chain = LLMChain(llm=llm, prompt=prompt)

    def run(self, input_msg: str) -> str:
        return self.chain.run(input_msg)



from oslui.command.base import BaseCommand


class LanguageCommand(BaseCommand):
    """
        LanguageCommand: language command
    """

    def __init__(self, content: str):
        super().__init__(content)



import os


def get_environment_variable(variable_name):
    try:
        value = os.environ[variable_name]
        return value
    except KeyError:
        raise Exception(f"environment variable {variable_name} does not exist")



import argparse
import logging
import os
import sys

from langchain.llms import OpenAI

curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)

from oslui.utils import get_environment_variable
from oslui.agent import TranslateAgent, QueryAgent, ChatAgent
from oslui.command import LanguageCommand, ShellCommand
from oslui.prompts import TRANSLATE_PROMPT, CHAT_PROMPT


def main():
    console_handler = logging.StreamHandler()
    console_handler.setLevel(logging.WARNING)
    logging.getLogger('').addHandler(console_handler)

    parser = argparse.ArgumentParser(
        description="Natural Language User Interface for Operating Systems")
    parser.add_argument("language_command", help="Natural language command")
    parser.add_argument("-q", "--query", action="store_true",
                        required=False, help="Query something")
    parser.add_argument("-g", "--gpt", action="store_true",
                        required=False, help="Chat to GPT")
    args = parser.parse_args()

    try:
        openai_api_key = get_environment_variable("OPENAI_API_KEY")
    except Exception as e:
        logging.error(str(e))
        exit(1)

    llm = OpenAI(temperature=0, openai_api_key=openai_api_key)

    if args.gpt:
        chat_agent = ChatAgent(llm, CHAT_PROMPT)
        result = chat_agent.run(args.language_command)
        print(result)
    elif args.query:
        query_agent = QueryAgent(llm)
        result = query_agent.run(args.language_command)
        print(result)
    else:
        trans_agent = TranslateAgent(llm, TRANSLATE_PROMPT)
        lang_cmd = LanguageCommand(args.language_command)
        sh_cmd = ShellCommand(trans_agent.run(lang_cmd.content))
        sh_cmd.modify()
        sh_cmd.run()


if __name__ == "__main__":
    main()



from typing import List

from langchain.agents import AgentType, initialize_agent, load_tools
from langchain.agents.agent import AgentExecutor
from langchain.llms.base import BaseLLM
from langchain.prompts import PromptTemplate
from langchain.tools.base import BaseTool

from oslui.agent.base import BaseAgent


class QueryAgent(BaseAgent):
    """
        QueryAgent: support query something in terminal
    """
    tools: List[BaseTool] = None
    agent: AgentExecutor = None

    def __init__(self, llm: BaseLLM, prompt: PromptTemplate = None, memory: PromptTemplate = None,
                 tools: List[str] = None):
        super().__init__(llm, prompt, memory)
        if tools:
            self.tools = load_tools(tools, llm=llm)
        else:
            self.tools = load_tools(["serpapi"], llm=llm)
        self.agent = initialize_agent(
            self.tools, llm, agent=AgentType.ZERO_SHOT_REACT_DESCRIPTION, verbose=False)

    def run(self, input_msg: str) -> str:
        return self.agent.run(input_msg)



from langchain.prompts import PromptTemplate

translate_tmp = \
    "You are a Linux expert, please translate the given natural language requirements into shell commands" \
    "##" \
    "natural language requirements 1:create a new git branch named develop" \
    "shell command 1:git branch develop" \
    "##" \
    "natural language requirements 2:将test.tar.gz解压缩到路径/aaa/bbb/ccc下" \
    "shell command 2:tar -xzvf test.tar.gz -C /aaa/bbb/ccc" \
    "##" \
    "natural language requirements 3:{language_command}" \
    "shell command 3:"

TRANSLATE_PROMPT = PromptTemplate(
    input_variables=["language_command"],
    template=translate_tmp,
)



from setuptools import setup, find_packages

setup(
    name='oslui',
    version='0.1.0',
    url='https://github.com/BalianWang/OSLUI',
    author='BalianWang',
    author_email='balian.wang1997@gmail.com',
    description='Natural Language User Interface for Operating Systems',
    packages=find_packages(),
    install_requires=[
        'openai',
        'langchain',
        'google-search-results',
    ],
    entry_points={
        'console_scripts': [
            'oslui = oslui.main:main',
        ],
    },
    license='MIT',
    python_requires='>=3'
)



from abc import ABC, abstractmethod

from langchain.llms.base import BaseLLM
from langchain.prompts import PromptTemplate


class BaseAgent(ABC):
    """
        BaseAgent: OS Agent base class
    """

    def __init__(self, llm: BaseLLM, prompt: PromptTemplate = None, memory: PromptTemplate = None):
        self.llm = llm
        self.prompt = prompt
        self.memory = memory

    @abstractmethod
    def run(self, input_msg: str) -> str:
        pass
